//----------------------------------------------------------------------------
// Indicator interface unit ver 1.1, ForexTester (c) Koshelev M.A.
//
// What's new:
// ver 1.2
//   1. Added new constant cl_GridColor - to paint indicator levels
//      with current grid color
//   2. Added new functions
//      IndicatorDigits - indicator precision
//      Timeframe - indicator timeframe
//
// ver 1.1
//   1. New interface function SetIndexSymbol
//   2. New draw style (TDrawStyle) ds_Symbol
//
// ver 1.0
//
//----------------------------------------------------------------------------
unit IndicatorInterfaceUnit;

interface

uses
  SysUtils, graphics, contnrs;

type
  //-------------------------------------------
  // currency information
  //-------------------------------------------
  PCurrencyInfo = ^TCurrencyInfo;
  TCurrencyInfo = packed record
    Symbol: PChar;                     // currency name
    Digits: integer;                   // number of digits after '.'
    spread: integer;                   // spread in pips
    Point: double;                     // minimal currency point
    lot: integer;                      // 1 lot cost
    curr: PChar;                       // lot currency
    SwapLong: double;                  // swap points long
    SwapShort: double;                 // swap points short
  end;


  //-------------------------------------------
  // index buffer
  //-------------------------------------------
  TIndexBuffer = class
  private
    fBufferIndex: integer;

    function    GetValue(index: integer): double;
    procedure   SetValue(index: integer; const Value: double);
  public
    constructor create(index: integer);
    function    count: integer;
    function    GetMax(index1, index2: integer): double;
    function    GetMin(index1, index2: integer): double;

    property    value[index: integer]: double read GetValue write SetValue; default;
  end;


  //-------------------------------------------
  // Buffers array
  //-------------------------------------------
  TBuffersArray = class
  private
    function    GetBuffer(index: integer): TIndexBuffer;
  public
    property    data[index: integer]: TIndexBuffer read GetBuffer; default;
  end;


  // timeframe
  TTimeframe =
    (PERIOD_M1  = 0,
     PERIOD_M5  = 1,
     PERIOD_M15 = 2,
     PERIOD_H1  = 3,
     PERIOD_H4  = 4,
     PERIOD_D1  = 5,
     PERIOD_W1  = 6);

  // option type
  TOptionType =
    (ot_Longword  = 0,
     ot_Integer   = 1,
     ot_double    = 2,
     ot_String    = 3,
     ot_Boolean   = 4,
     ot_EnumType  = 5,
     ot_Timeframe = 6,
     ot_Currency  = 7,
     ot_LineStyle = 8,
     ot_Separator = 9);

  // indicator window
  TOutputWindow =
    (ow_ChartWindow    = 0,
     ow_SeparateWindow = 1);

  // drawing type
  TDrawStyle =
    (ds_Line       = 0,
     ds_Histogram  = 1,
     ds_Fill       = 2,
     ds_Symbol     = 3,
     ds_None       = 10
     );


const
  // current color of the grid (AddLevel procedure)
  cl_GridColor = $01000000;


// Get currency information
// Symbol - currency name
// info - pointer to TCurrencyInfo record
// result: true if success
function  GetCurrencyInfo(Symbol: string; var info: PCurrencyInfo): boolean;

// Register option
// OptionName - name of option
// OptionType - see TOptionType enumeration
// option - pointer to first byte
procedure RegOption(OptionName: string; OptionType: TOptionType; var option);

// Add separator to options dialog
// text - separator caption
procedure AddSeparator(text: string);

// Add option value (for drop down box options)
// OptionName - name of previously registered option
// value - new value in drop down box
procedure AddOptionValue(OptionName, value: string);

// Set option range (for integer, longword and double types)
// OptionName - name of previously registered option
// LowValue - lowest value in range
// HighValue - highest value in range
// note: in dialog box after entering option value it will be checked
//       to be in range LowValue <= value <= HighValue
procedure SetOptionRange(OptionName: string; LowValue, HighValue: double);

// Set option digits (for double options) defines number of digits after point
// OptionName - name of option
// digits - number of digits after point
procedure SetOptionDigits(OptionName: string; digits: word);

// Print text in "Journal" window in ForexTester
// s - text
procedure Print(s: string);

// Set indicator short name
// name - short name
procedure IndicatorShortName(name: string);

// Set number of index buffers
// count - number of buffers
procedure IndicatorBuffers(count: integer);

// Create index buffer
// result - buffer id
function  CreateIndexBuffer: TIndexBuffer;

// Set index buffer
// index - number of index buffer slot
// buffer - buffer id
procedure SetIndexBuffer(index: integer; buffer: TIndexBuffer);

// Set index buffer style
// index - number of index buffer slot
// _type - paint type
// style - line style
// width - line width
// color - line color
procedure SetIndexStyle(index: integer; DrawStyle: TDrawStyle; style: TPenStyle;
            width: integer; color: TColor);

// Set index symbol
// index - number of an index buffer slot
// symbol - number of symbol in Wingdings font
// xoffs - x offset from center point of the symbol in pixels
// yoffs - y offset from center point of the symbol in pixels
procedure SetIndexSymbol(index, symbol: integer; xoffs: integer = 0;
            yoffs: integer = 0);

// Set indicator output window
// aWindow - where to paint indicator (ow_ChartWindow, ow_SeparateWindow)
procedure SetOutputWindow(aWindow: TOutputWindow);

// Set fixed maximum/minimum for indicator window
// aMin - minimum value
// aMax - maximum value
procedure SetFixedMinMaxValues(aMin, aMax: double);

// Add level line to indicator window
// value - level value
// style - line style
// width - line width
// color - line color
procedure AddLevel(value: double; style: TPenStyle; width: integer; color: TColor);

// Set empty value that will not be painted
// value - new empty value
procedure SetEmptyValue(value: double);

// Currency name
function  Symbol: string;

// Currency digits after point
function  Digits: integer;

// Minimal currency point
function  Point: double;

// Get open value
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iOpen(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;

// Get close value
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iClose(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;

// Get high value
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iHigh(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;

// Get low value
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iLow(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;

// Get volume
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iVolume(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;

// Get time of bar
// Symbol - requested currency
// TimeFrame - requested timeframe
// shift - shift in bars from last bar
function  iTime(Symbol: string; TimeFrame: TTimeframe; shift: longword): TDateTime;

// Get number of bars
// Symbol - requested currency
// TimeFrame - requested timeframe
function  iBars(Symbol: string; TimeFrame: TTimeframe): integer;

// Get open value in default currency and timeframe
// shift - shift in bars from last bar
function  Open(shift: longword): double;

// Get close value in default currency and timeframe
// shift - shift in bars from last bar
function  Close(shift: longword): double;

// Get high value in default currency and timeframe
// shift - shift in bars from last bar
function  High(shift: longword): double;

// Get low value in default currency and timeframe
// shift - shift in bars from last bar
function  Low(shift: longword): double;

// Get volume in default currency and timeframe
// shift - shift in bars from last bar
function  Volume(shift: longword): double;

// Get time of bar in default currency and timeframe
// shift - shift in bars from last bar
function  Time(shift: longword): TDateTime;

// Get number of bars in default currency and timeframe
function  Bars: integer;

// Set indicator precision (number of digits after .)
// digits - number of digits after point
procedure IndicatorDigits(digits: integer);

// Get indicator timeframe
function  Timeframe: TTimeframe;


var
  buff: TBuffersArray;


implementation

type
  // indicator interface procedures
  TGetCurrencyInfoFunc = function(Symbol: PChar; var info: PCurrencyInfo): boolean of object; stdcall;
  TSetPropertyProc = procedure(PropertyID: integer; value: OLEVariant) of object; stdcall;
  TRegOptionProc = procedure(OptionName: PChar; OptionType: integer; OptPtr: pointer) of object; stdcall;
  TAddOptionValueProc = procedure(OptionName, value: PChar) of object; stdcall;
  TSetOptionRange = procedure(OptionName: PChar; LowValue, HighValue: double) of object; stdcall;
  TSetOptionDigitsProc = procedure(OptionName: PChar; digits: word) of object; stdcall;
  TPrintProc = procedure(s: PChar) of object; stdcall;
  TGetDoubleFunc = function: double of object; stdcall;
  TGetPCharFunc = function: PChar of object; stdcall;
  TGetIntegerFunc = function: integer of object; stdcall;
  TGet_iOHLCVFunc = function(Symbol: PChar; TimeFrame, shift: longword): double of object; stdcall;
  TGet_iTimeFunc = function(Symbol: PChar; TimeFrame, shift: longword): TDateTime of object; stdcall;
  TGet_iBarsFunc = function(Symbol: PChar; TimeFrame: longword): longword of object; stdcall;
  TGetOHLCVFunc = function(shift: longword): double of object; stdcall;
  TGetTimeFunc = function(shift: longword): TDateTime of object; stdcall;
  TGetBarsFunc = function: longword of object; stdcall;
  TIndicatorShortNameProc = procedure(name: PChar) of object; stdcall;
  TIndicatorBuffersProc = procedure(count: integer) of object; stdcall;
  TCreateIndexBufferFunc = function: integer of object; stdcall;
  TSetIndexBufferProc = procedure(index, buffer: integer) of object; stdcall;
  TSetIndexStyleProc = procedure(index, _type, _style, width, clr: integer) of object; stdcall;
  TGetBufferValueFunc = function(buffer: integer; index: longword): double of object; stdcall;
  TSetBufferValueProc = procedure(buffer: integer; index: longword; value: double) of object;
  TGetBufferCountFunc = function(buffer: integer): longword of object; stdcall;
  TGetBufferMaxFunc = function(buffer: integer; index1, index2: longword): double of object; stdcall;
  TGetBufferMinFunc = function(buffer: integer; index1, index2: longword): double of object; stdcall;
  TSetOutputWindowProc = procedure(aWindow: integer) of object; stdcall;
  TSetFixedMinMaxValuesProc = procedure(aMin, aMax: double) of object; stdcall;
  TAddLevelProc = procedure(value: double; _style, width, clr: integer) of object; stdcall;
  TSetEmptyValueProc = procedure(value: double) of object; stdcall;
  TSetIndexSymbolProc = procedure(index, symbol, xoffs, yoffs: integer) of object; stdcall;
  TIndicatorDigitsProc = procedure(digits: integer) of object; stdcall;

  // pointers to interface procedures
  PInterfaceProcRec = ^TInterfaceProcRec;
  TInterfaceProcRec = packed record
    dwSize: longword;

    GetCurrencyInfo: TGetCurrencyInfoFunc;
    RegOption: TRegOptionProc;
    AddOptionValue: TAddOptionValueProc;
    SetOptionRange: TSetOptionRange;
    SetOptionDigits: TSetOptionDigitsProc;
    Print: TPrintProc;
    IndicatorShortName: TIndicatorShortNameProc;
    IndicatorBuffers: TIndicatorBuffersProc;
    CreateIndexBuffer: TCreateIndexBufferFunc;
    SetIndexBuffer: TSetIndexBufferProc;
    SetIndexStyle: TSetIndexStyleProc;
    SetOutputWindow: TSetOutputWindowProc;
    SetFixedMinMaxValues: TSetFixedMinMaxValuesProc;
    AddLevel: TAddLevelProc;
    SetEmptyValue: TSetEmptyValueProc;

    GetBufferValue: TGetBufferValueFunc;
    SetBufferValue: TSetBufferValueProc;
    GetBufferCount: TGetBufferCountFunc;
    GetBufferMax: TGetBufferMaxFunc;
    GetBufferMin: TGetBufferMinFunc;

    Symbol: TGetPCharFunc;
    Digits: TGetIntegerFunc;
    Point: TGetDoubleFunc;

    iOpen: TGet_iOHLCVFunc;
    iClose: TGet_iOHLCVFunc;
    iHigh: TGet_iOHLCVFunc;
    iLow: TGet_iOHLCVFunc;
    iVolume: TGet_iOHLCVFunc;
    iTime: TGet_iTimeFunc;
    iBars: TGet_iBarsFunc;

    Open: TGetOHLCVFunc;
    Close: TGetOHLCVFunc;
    High: TGetOHLCVFunc;
    Low: TGetOHLCVFunc;
    Volume: TGetOHLCVFunc;
    Time: TGetTimeFunc;
    Bars: TGetBarsFunc;

    // extentions (dwSize > 300)
    SetIndexSymbol: TSetIndexSymbolProc;

    // extensions (dwSize > 308)
    IndicatorDigits: TIndicatorDigitsProc;
    Timeframe: TGetIntegerFunc;
  end;


var
  IntrfProcsRec: TInterfaceProcRec;
  buffers: TObjectList;

{-----Get currency information-----------------------------------------------}
function GetCurrencyInfo(Symbol: string; var info: PCurrencyInfo): boolean;
begin
  result := IntrfProcsRec.GetCurrencyInfo(PChar(Symbol), info)
end;

{-----Register option--------------------------------------------------------}
procedure RegOption(OptionName: string; OptionType: TOptionType; var option);
begin
  IntrfProcsRec.RegOption(PChar(OptionName), integer(OptionType), @option);
end;

{-----Add separator----------------------------------------------------------}
procedure AddSeparator(text: string);
begin
  IntrfProcsRec.RegOption(PChar(text), integer(ot_Separator), nil);
end;

{-----Add option value (for drop down box options)---------------------------}
procedure AddOptionValue(OptionName, value: string);
begin
  IntrfProcsRec.AddOptionValue(PChar(OptionName), PChar(value));
end;

{-----Set option range (for integer, longword and double types)--------------}
procedure SetOptionRange(OptionName: string; LowValue, HighValue: double);
begin
  IntrfProcsRec.SetOptionRange(PChar(OptionName), LowValue, HighValue);
end;

{-----Set option digits------------------------------------------------------}
procedure SetOptionDigits(OptionName: string; digits: word);
begin
  IntrfProcsRec.SetOptionDigits(PChar(OptionName), digits);
end;

{-----Print text in "Journal" window in ForexTester--------------------------}
procedure Print(s: string);
begin
  IntrfProcsRec.Print(PChar(s));
end;

{-----Set indicator short name-----------------------------------------------}
procedure IndicatorShortName(name: string);
begin
  IntrfProcsRec.IndicatorShortName(PChar(name));
end;

{-----Set number of index buffers--------------------------------------------}
procedure IndicatorBuffers(count: integer);
begin
  IntrfProcsRec.IndicatorBuffers(count);
end;

{-----Create index buffer----------------------------------------------------}
function  CreateIndexBuffer: TIndexBuffer;
begin
  result := TIndexBuffer.create(IntrfProcsRec.CreateIndexBuffer);
  buffers.Add(result);
end;

{-----Set index buffer-------------------------------------------------------}
procedure SetIndexBuffer(index: integer; buffer: TIndexBuffer);
begin
  IntrfProcsRec.SetIndexBuffer(index, buffer.fBufferIndex);
end;

{-----Set index buffer style-------------------------------------------------}
procedure SetIndexStyle(index: integer; DrawStyle: TDrawStyle; style: TPenStyle;
  width: integer; color: TColor);
begin
  IntrfProcsRec.SetIndexStyle(index, integer(DrawStyle), integer(style), width, integer(color));
end;

{-----Set index buffer symbol------------------------------------------------}
procedure SetIndexSymbol(index, symbol: integer; xoffs: integer = 0;
  yoffs: integer = 0);
begin
  if assigned(IntrfProcsRec.SetIndexSymbol) then
    IntrfProcsRec.SetIndexSymbol(index, symbol, xoffs, yoffs);
end;

{-----Currency name----------------------------------------------------------}
function  Symbol: string;
begin
  result := IntrfProcsRec.Symbol;
end;

{-----Currency digits after point--------------------------------------------}
function  Digits: integer;
begin
  result := IntrfProcsRec.Digits;
end;

{-----Minimal currency point-------------------------------------------------}
function  Point: double;
begin
  result := IntrfProcsRec.Point;
end;

{-----Get open value---------------------------------------------------------}
function  iOpen(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;
begin
  result := IntrfProcsRec.iOpen(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get close value--------------------------------------------------------}
function  iClose(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;
begin
  result := IntrfProcsRec.iClose(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get high value---------------------------------------------------------}
function  iHigh(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;
begin
  result := IntrfProcsRec.iHigh(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get low value----------------------------------------------------------}
function  iLow(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;
begin
  result := IntrfProcsRec.iLow(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get volume-------------------------------------------------------------}
function  iVolume(Symbol: string; TimeFrame: TTimeframe; shift: longword): double;
begin
  result := IntrfProcsRec.iVolume(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get time of bar--------------------------------------------------------}
function  iTime(Symbol: string; TimeFrame: TTimeframe; shift: longword): TDateTime;
begin
  result := IntrfProcsRec.iTime(PChar(Symbol), longword(TimeFrame), shift);
end;

{-----Get number of bars-----------------------------------------------------}
function  iBars(Symbol: string; TimeFrame: TTimeframe): integer;
begin
  result := integer(IntrfProcsRec.iBars(PChar(Symbol), longword(TimeFrame)));
end;

{-----Get open value in default currency and timeframe-----------------------}
function  Open(shift: longword): double;
begin
  result := IntrfProcsRec.Open(shift);
end;

{-----Get close value in default currency and timeframe----------------------}
function  Close(shift: longword): double;
begin
  result := IntrfProcsRec.Close(shift);
end;

{-----Get high value in default currency and timeframe-----------------------}
function  High(shift: longword): double;
begin
  result := IntrfProcsRec.High(shift);
end;

{-----Get low value in default currency and timeframe------------------------}
function  Low(shift: longword): double;
begin
  result := IntrfProcsRec.Low(shift);
end;

{-----Get volume in default currency and timeframe---------------------------}
function  Volume(shift: longword): double;
begin
  result := IntrfProcsRec.Volume(shift);
end;

{-----Get time of bar in default currency and timeframe----------------------}
function  Time(shift: longword): TDateTime;
begin
  result := IntrfProcsRec.Time(shift);
end;

{-----Get number of bars in default currency and timeframe-------------------}
function  Bars: integer;
begin
  result := integer(IntrfProcsRec.Bars);
end;

{-----Replace PChar string with new------------------------------------------}
procedure ReplaceStr(var dest: PChar; source: PChar); stdcall;
begin
  FreeMem(dest);
  GetMem(dest, StrLen(source) + 1);
  StrCopy(dest, source);
end;

{-----Set indicator output window--------------------------------------------}
procedure SetOutputWindow(aWindow: TOutputWindow);
begin
  IntrfProcsRec.SetOutputWindow(integer(aWindow));
end;

{-----Set fixed maximum/minimum for indicator window-------------------------}
procedure SetFixedMinMaxValues(aMin, aMax: double);
begin
  IntrfProcsRec.SetFixedMinMaxValues(aMin, aMax);
end;

{-----Add level line to indicator window-------------------------------------}
procedure AddLevel(value: double; style: TPenStyle; width: integer; color: TColor);
begin
  IntrfProcsRec.AddLevel(value, integer(style), width, integer(color));
end;

{-----Set empty value that will not be painted-------------------------------}
procedure SetEmptyValue(value: double);
begin
  IntrfProcsRec.SetEmptyValue(value);
end;

{-----Set indicator precision------------------------------------------------}
procedure IndicatorDigits(digits: integer);
begin
  if assigned(IntrfProcsRec.IndicatorDigits) then
    IntrfProcsRec.IndicatorDigits(digits);
end;

{-----Get indicator timeframe------------------------------------------------}
function  Timeframe: TTimeframe;
begin
  if assigned(IntrfProcsRec.Timeframe) then
    result := TTimeframe(IntrfProcsRec.Timeframe)
  else
    result := PERIOD_M1;
end;


{===== TIndexBuffer =========================================================}
constructor TIndexBuffer.create(index: integer);
begin
  fBufferIndex := index;
end;

{-----Get count--------------------------------------------------------------}
function TIndexBuffer.count: integer;
begin
  result := integer(IntrfProcsRec.GetBufferCount(fBufferIndex));
end;

{-----Get max value in range-------------------------------------------------}
function TIndexBuffer.GetMax(index1, index2: integer): double;
var
  cnt: integer;
begin
  cnt := count - 1;
  index1 := cnt - index1;
  index2 := cnt - index2;
  if index1 < 0 then
    index1 := 0;
  if index2 < 0 then
    index2 := 0;
  result := IntrfProcsRec.GetBufferMax(fBufferIndex, index1, index2);
end;

{-----Get min value in range-------------------------------------------------}
function TIndexBuffer.GetMin(index1, index2: integer): double;
var
  cnt: integer;
begin
  cnt := count - 1;
  index1 := cnt - index1;
  index2 := cnt - index2;
  if index1 < 0 then
    index1 := 0;
  if index2 < 0 then
    index2 := 0;
  result := IntrfProcsRec.GetBufferMin(fBufferIndex, index1, index2);
end;

{-----Get value by index-----------------------------------------------------}
function TIndexBuffer.GetValue(index: integer): double;
begin
  index := count - 1 - index;
  if index < 0 then
    index := 0;
  result := IntrfProcsRec.GetBufferValue(fBufferIndex, index);
end;

{-----Set value by index-----------------------------------------------------}
procedure TIndexBuffer.SetValue(index: integer; const Value: double);
begin
  index := count - 1 - index;
  if index < 0 then
    index := 0;
  IntrfProcsRec.SetBufferValue(fBufferIndex, index, value);
end;

{-----Get buffer-------------------------------------------------------------}
function TBuffersArray.GetBuffer(index: integer): TIndexBuffer;
var
  i: integer;
begin
  for i:=0 to buffers.Count - 1 do
    if TIndexBuffer(buffers[i]).fBufferIndex = index then
      begin
        result := TIndexBuffer(buffers[i]);
        exit;
      end;
  result := nil;
end;



exports

IntrfProcsRec, ReplaceStr;


initialization

fillchar(IntrfProcsRec, sizeof(IntrfProcsRec), 0);
IntrfProcsRec.dwSize := sizeof(IntrfProcsRec);
buffers := TObjectList.Create;
buff := TBuffersArray.Create;

finalization

buffers.Free;
buff.Free;

end.
